#ifndef _MODELOUTPUT_H
#define _MODELOUTPUT_H

#include <math.h>
#include <vector>

const int ROW_MAJOR = 0;
const int COL_MAJOR = 1;

// the number of entries has to be the same for each list or Origin will not read them in properly
// that means you must pad with zeros (only true for row major?)
class ModelOutput
{
  public:
        ModelOutput (char *pfilename, int pnum_lists, int pnum_entries, int prow_major)
	  : filename(pfilename), num_lists(pnum_lists), num_entries(pnum_entries),
            data(pnum_lists, std::vector<float> (pnum_entries)),
            current_list(0), current_entry(0), row_major(prow_major)
        {
          // size of "data" is not dynamical allocated on purpose!!!
	}

       ~ModelOutput (void)
	{
          FILE *outfile = NULL;

	  if ((outfile = fopen(filename, "w")) == NULL)
	    Printf ("Cannot open file %s\n", filename);		

          // write output depending on the table type
          if (row_major == ROW_MAJOR)
          {
            // row-major format
            for (int i = 0; i < data.size(); i++)
            {
              for (int j = 0; j < data[i].size(); j++)
                if (std::isnan(data[i][j]))
                  fprintf (outfile, "-- ");
                else
                  fprintf (outfile, "%g ", data[i][j]);
              fprintf (outfile, "\n");
            }
          }
          else
          {
            // column-major format
            for (int i = 0; i < num_entries; i++)
            {
              for (int j = 0; j < data.size(); j++)
                if (std::isnan(data[j][i]))
                  fprintf (outfile, "-- ");
                else
                  fprintf (outfile, "%g ", data[j][i]);
              fprintf (outfile, "\n");
            }
          }

	  if (outfile != NULL)
	    fclose (outfile);
	}

	// write 'value' as next entry in current list
        void write (float value)
	{
          if (current_list < num_lists)
          {
            if (current_entry < num_entries)
            {
              data[current_list][current_entry] = value;
              ++current_entry;
            }
            else
	      Printf ("Warning: output to %s exceeds specified number of entries per list\n", filename);
          }
	}
	
	// new list of data (i.e. a new row or column depending on the type)
	void newList (void)
	{
	  if (current_list < num_lists-1)
            ++current_list;
	  else
	    Printf ("Warning: output to %s exceeds specified number of lists\n", filename);

          current_entry = 0;
	}
	
	void reset (void)
	{
	  current_list = 0;
          current_entry = 0;

          for (int i = 0; i < data.size(); i++)
            data[i].assign(num_entries, NAN);
	}

  private:
	char *filename;
	int num_lists, num_entries, current_list, current_entry;
        int row_major; // either row major or column major format writting
        std::vector<std::vector<float> > data;
};

#endif
